#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# Board support for this chinese Kintex 420T board by "SITLINV FPGA Board Store"
# https://www.aliexpress.com/item/1005001631827738.html

# IOs ----------------------------------------------------------------------------------------------

def _get_io(io_standard):
    _io = [
        # Clk / Rst
        ("clk100",      0, Pins("U24"), io_standard),

        ("diffclk100",  0,
            Subsignal("p", Pins("U22"), IOStandard("LVDS_25")),
            Subsignal("n", Pins("U23"), IOStandard("LVDS_25"))
        ),

        # Leds
        ("user_led_n", 0, Pins("A27"),  IOStandard("LVCMOS15")),
        ("user_led_n", 1, Pins("E24"),  IOStandard("LVCMOS15")),
        ("user_led_n", 2, Pins("G24"),  IOStandard("LVCMOS15")),
        ("user_led_n", 3, Pins("H21"),  IOStandard("LVCMOS15")),
        ("user_led_n", 4, Pins("G27"),  IOStandard("LVCMOS15")),
        ("user_led_n", 5, Pins("H26"),  IOStandard("LVCMOS15")),
        ("user_led_n", 6, Pins("H25"),  IOStandard("LVCMOS15")),
        ("user_led_n", 7, Pins("H24"),  IOStandard("LVCMOS15")),

        # Buttons
        ("user_btn_n",  0, Pins("Y23"), IOStandard("LVCMOS15")),
        ("cpu_reset_n", 0, Pins("J24"), IOStandard("LVCMOS15")), # J4 jumper 2.5V or 3.3V

        # I2C / AT24C04
        ("i2c", 0,
            Subsignal("scl", Pins("C17")),
            Subsignal("sda", Pins("C16")),
            IOStandard("LVCMOS33")
        ),

        # Serial
        ("serial", 0,
            Subsignal("tx",  Pins("D16")),  # CH340_TX
            Subsignal("rx",  Pins("D17")),  # CH340_RX
            IOStandard("LVCMOS33")
        ),

        # DDR3 SDRAM near SFP ports
        ("ddram", 0,
            Subsignal("a",     Pins(
                "F28 E29 F26 D29 B29 C30 A30 B28 C29 B30 E30 E26 A28 H29 F25"),
                IOStandard("SSTL15")),
            Subsignal("ba",    Pins("F30 G28 E28"), IOStandard("SSTL15")),
            Subsignal("ras_n", Pins("H27"),  IOStandard("SSTL15")),
            Subsignal("cas_n", Pins("G30"),  IOStandard("SSTL15")),
            Subsignal("we_n",  Pins("G29"),  IOStandard("SSTL15")),
            Subsignal("cs_n",  Pins("H30"),  IOStandard("SSTL15")),
            Subsignal("dm",      Pins(
                "B22 E19 F22 K19 M23 P18 P26 N29"),
                IOStandard("SSTL15")),
            Subsignal("dq",      Pins(
                "A21 A22 A23 B23 B19 C19 A20 B20 C21 D21 C22 D22 E18 D18 E20 E21 " + \
                "G18 F18 G20 F20 H20 G22 G23 F23 L18 J18 J19 K20 J22 H22 K23 J23 " + \
                "N24 N22 P24 P23 L20 M22 M24 N25 M17 N19 N17 P17 N20 N21 P21 P19 " + \
                "K26 K25 L26 L25 M25 N26 P28 P27 L30 M29 P29 R29 K28 K29 K30 M28 "),
                IOStandard("SSTL15")),
            Subsignal("dqs_p",   Pins("B18 E23 H19 K21 L23 M18 N27 N30"),
                IOStandard("DIFF_SSTL15")),
            Subsignal("dqs_n",   Pins("A18 D23 G19 J21 K24 M19 M27 M30"),
                IOStandard("DIFF_SSTL15")),
            Subsignal("clk_p",   Pins("J26"),  IOStandard("DIFF_SSTL15"), Misc("IO_BUFFER_TYPE=NONE")),
            Subsignal("clk_n",   Pins("J27"),  IOStandard("DIFF_SSTL15"), Misc("IO_BUFFER_TYPE=NONE")),
            Subsignal("cke",     Pins("G25"),  IOStandard("SSTL15")),
            Subsignal("odt",     Pins("J28"),  IOStandard("SSTL15")),
            Subsignal("reset_n", Pins("F27"),  IOStandard("LVCMOS15")),
            Misc("SLEW=FAST"),
        ),

        # DDR3 SDRAM near power supply
        ("ddram", 1,
            Subsignal("a",     Pins(
                "AG22 AJ23 AF22 AJ26 AG23 AD23 AF23 AJ24 AE23 AB23 AJ22 AK25 AD21 AD22 AK24"),
                IOStandard("SSTL15")),
            Subsignal("ba",    Pins("AK23 AF21 AC21"), IOStandard("SSTL15")),
            Subsignal("ras_n", Pins("AF20"), IOStandard("SSTL15")),
            Subsignal("cas_n", Pins("AK21"), IOStandard("SSTL15")),
            Subsignal("we_n",  Pins("AJ21"), IOStandard("SSTL15")),
            Subsignal("cs_n",  Pins("AE21"), IOStandard("SSTL15")),
            Subsignal("dm",      Pins(
                "AA28 AA27 AE28 AH30 AB18 AJ19 AD14 AK16"),
                IOStandard("SSTL15")),
            Subsignal("dq",      Pins(
                "W29   Y29 AB30 AB29  W28  W26  Y28 AB28 AA25 AD27 AB24 AC24  Y26  Y25 AA26 AC26 " + \
                "AD29 AE30 AE29 AF30 AD28 AC27 AF28 AF27 AG30 AG29 AH29 AJ29 AK30 AK29 AK28 AG27 " + \
                "AD18 AD19 AA18  Y18 AE18  Y19 AB17 AA17 AH20 AH19 AG19 AF18 AJ18 AK18 AJ17 AJ16 " + \
                "AF16 AE16 AE15 AF15 AC15 AB15 AC14 AB14 AH17 AH16 AK14 AJ14 AF17 AG17 AH15 AH14 "),
                IOStandard("SSTL15")),
            Subsignal("dqs_p",   Pins("Y30 AB25 AC29 AJ27 AC17 AK19 AC16 AG14"),
                IOStandard("DIFF_SSTL15")),
            Subsignal("dqs_n",   Pins("AA30 AC25 AC30 AJ28 AD17 AK20 AD16 AG15"),
                IOStandard("DIFF_SSTL15")),
            Subsignal("clk_p",   Pins("AA22"),  IOStandard("DIFF_SSTL15")),
            Subsignal("clk_n",   Pins("AA23"),  IOStandard("DIFF_SSTL15")),
            Subsignal("cke",     Pins("AB22"),  IOStandard("SSTL15")),
            Subsignal("odt",     Pins("AG20"),  IOStandard("SSTL15")),
            Subsignal("reset_n", Pins("Y21"),   IOStandard("LVCMOS15")),
            Misc("SLEW=FAST"),
        ),

        # Sata
        ("sata", 0,
            Subsignal("rx_p", Pins("C12")),
            Subsignal("rx_n", Pins("C11")),
            Subsignal("tx_p", Pins("A12")),
            Subsignal("tx_n", Pins("A11")),
        ),
        ("sata", 1,
            Subsignal("rx_p", Pins("E12")),
            Subsignal("rx_n", Pins("E11")),
            Subsignal("tx_p", Pins("B10")),
            Subsignal("tx_n", Pins("B9")),
        ),

        # PCIe
        ("pcie_x1", 0,
            Subsignal("rst_n", Pins("W21"), io_standard),
            Subsignal("clk_p", Pins("T6")),
            Subsignal("clk_n", Pins("T5")),
            Subsignal("rx_p",  Pins("P6")),
            Subsignal("rx_n",  Pins("P5")),
            Subsignal("tx_p",  Pins("N4")),
            Subsignal("tx_n",  Pins("N3"))
        ),
        ("pcie_x2", 0,
            Subsignal("rst_n", Pins("W21"), io_standard),
            Subsignal("clk_p", Pins("T6")),
            Subsignal("clk_n", Pins("T5")),
            Subsignal("rx_p",  Pins("")),
            Subsignal("rx_p",  Pins("P6 R4")),
            Subsignal("rx_n",  Pins("P5 R3")),
            Subsignal("tx_p",  Pins("N4 P2")),
            Subsignal("tx_n",  Pins("N3 P1"))
        ),
        ("pcie_x4", 0,
            Subsignal("rst_n", Pins("W21"), io_standard),
            Subsignal("clk_p", Pins("T6")),
            Subsignal("clk_n", Pins("T5")),
            Subsignal("rx_p",  Pins("P6 R4 U4 V6")),
            Subsignal("rx_n",  Pins("P5 R3 U3 V5")),
            Subsignal("tx_p",  Pins("N4 P2 T2 V2")),
            Subsignal("tx_n",  Pins("N3 P1 T1 V1"))
        ),
        ("pcie_x8", 0,
            Subsignal("rst_n", Pins("W21"), io_standard),
            Subsignal("clk_p", Pins("T6")),
            Subsignal("clk_n", Pins("T5")),
            Subsignal("rx_p",  Pins("P6 R4 U4 V6 W4  Y6 AA4 AB6")),
            Subsignal("rx_n",  Pins("P5 R3 U3 V5 W3  Y5 AA3 AB5")),
            Subsignal("tx_p",  Pins("N4 P2 T2 V2 Y2 AB2 AD2 AF2")),
            Subsignal("tx_n",  Pins("N3 P1 T1 V1 Y1 AB1 AD1 AF1"))
        ),


        # SFP A
        ("sfp_a", 0,
            Subsignal("txp", Pins("A8")),
            Subsignal("txn", Pins("A7")),
            Subsignal("rxp", Pins("D10")),
            Subsignal("rxn", Pins("D9")),
            Subsignal("sda", Pins("C15"), io_standard),
            Subsignal("scl", Pins("A15"), io_standard),
        ),
        ("sfp_a_tx", 0,
            Subsignal("p", Pins("A8")),
            Subsignal("n", Pins("A7"))
        ),
        ("sfp_a_rx", 0,
            Subsignal("p", Pins("D10")),
            Subsignal("n", Pins("D9"))
        ),
        ("sfp_a_tx_disable_n", 0, Pins("Y20"), io_standard),

        # SFP B
        ("sfp_b", 0,
            Subsignal("txp", Pins("C8")),
            Subsignal("txn", Pins("C7")),
            Subsignal("rxp", Pins("F10")),
            Subsignal("rxn", Pins("F9")),
            Subsignal("sda", Pins("C14"), io_standard),
            Subsignal("scl", Pins("B14"), io_standard),
        ),
        ("sfp_b_tx", 0,
            Subsignal("p", Pins("C8")),
            Subsignal("n", Pins("C7"))
        ),
        ("sfp_b_rx", 0,
            Subsignal("p", Pins("F10")),
            Subsignal("n", Pins("F9"))
        ),
        ("sfp_b_tx_disable_n", 0, Pins("D14"), io_standard),
    ]

    return _io

# Connectors ---------------------------------------------------------------------------------------

#
#         Connector layout on the board
#   ┌────────────────────────────────────────┐
#   │    2                            80     │
#   │    ┌──────────────────────────────┐    │
#   └──┐ └──────────────────────────────┘ ┌──┘
#      │ 1                            79  │
#      └──────────────────────────────────┘
#
_connectors = [
        # Connector on the SFP side
        ("BTB_A", {
          # 1:  "GND",   2: "GND",
            3:  "A16",   4: "B24",
            5:  "B17",   6: "D24",
          # 7:  "GND",   8: "GND",
            9:  "E16",  10: "A14",
            11: "F16",  12: "B15",
            13: "R25",  14: "U30",
            15: "R24",  16: "U29",
          # 17: "GND",  18: "GND",
            19: "R21",  20: "T27",
            21: "R20",  22: "R26",
            23: "T23",  24: "U28",
            25: "R23",  26: "U27",
          # 27: "GND",  28: "GND",
            29: "T18",  30: "V25",
            31: "T17",  32: "V24",
            33: "V20",  34: "R19",
            35: "U20",  36: "R18",
          # 37: "GND",  38: "GND",
            39: "W23",  40: "T21",
            41: "W22",  42: "T20",
            43: "U18",  44: "V19",
            45: "U17",  46: "U19",
          # 47: "GND",  48: "GND",
            49: "T26",  50: "W17",
            51: "T25",  52: "V17",
            53: "V22",  54: "W19",
            55: "V21",  56: "W18",
          # 57: "GND",  58: "GND",
            59: "C24",  60: "T22",
            61: "D26",  62: "V30",
            63: "C27",  64: "U25",
            65: "B27",  66: "AF25",
          # 67: "GND",  68: "GND"x
            69: "Y24",  70: "AH26",
            71: "AE26", 72: "AG25",
            73: "AD26", 74: "AH25",
          # 75: "GND",  76: "GND"
          # 77: "NC",   78: "3V3"
          # 79: "NC",   80: "3V3"
        }),

        # Connector on the power side
        ("BTB_B", {
          # 1:  "GND",    2: "GND",
            3:  "AJ11",   4: "AK9",
            5:  "AJ12",   6: "AK10",
          # 7:  "GND",    8: "GND",
            9:  "AJ7",    10: "AG11",
            11: "AJ8",    12: "AG12",
          # 13: "GND",    14: "GND",
            15: "AF9",    16: "AG7",
            17: "AF10",   18: "AG8",
          # 19: "GND",    20: "GND",
            21: "AE11",   22: "AH9",
            23: "AE12",   24: "AH10",
          # 25: "GND",    26: "GND",
            27: "AE8",    28: "AF6",
            29: "AE7",    30: "AF5",
          # 31: "GND",    32: "GND",
            33: "AG3",    34: "AK5",
            35: "AG4",    36: "AK6",
          # 37: "GND",    38: "GND",
            39: "AE3",    40: "AH5",
            41: "AE4",    42: "AH6",
          # 43: "GND",    44: "GND",
            45: "AK1",    46: "AJ3",
            47: "AK2",    48: "AJ4",
         #  49: "GND",    50: "GND",
            51: "AC3",    52: "AH1",
            53: "AC4",    54: "AH2",
          # 55: "GND",    56: "GND",
                          58: "AC19",
            59: "L17",    60: "AB19",
          # 61: "GND",    62: "GND",
            63: "AC20",   64: "AB20",
            65: "AE20",   66: "AA20",
          # 67: "GND",    68: "GND",
            69: "W24",    70: "Y20",
                          72: "AA21",
          # 73: "GND",    74: "GND",
          # 75: "NC",     76: "GND",
          # 77: "VCC12V", 78: "VCC3.3V",
          # 79: "VCC12V", 80: "VCC3.3V",
        }),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, io_voltage="3.3V"):
        assert io_voltage in ["2.5V", "3.3V"], "io_voltage must be '2.5V' or '3.3V' acording to the board jumper"
        io_standard = IOStandard("LVCMOS33") if io_voltage == "3.3V" else IOStandard("LVCMOS25")
        _io = _get_io(io_standard)

        Xilinx7SeriesPlatform.__init__(self, "xc7k420t-ffg901-2", _io, _connectors, toolchain="vivado")
        self.add_platform_command("""
        set_property CONFIG_VOLTAGE 3.3 [current_design]
        set_property CFGBVS VCCO [current_design]""")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 11]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 12]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 13]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 16]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 17]")

        self.toolchain.bitstream_commands = [
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]",
            "set_property BITSTREAM.CONFIG.CCLK_TRISTATE TRUE [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGRATE 66 [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_32BIT_ADDR YES [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_FALL_EDGE YES [current_design]",
            "set_property BITSTREAM.CONFIG.UNUSEDPIN PULLUP [current_design]",
            ]
        self.toolchain.additional_commands = ["write_cfgmem -force -format bin -interface spix4 -size 32 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft232.cfg", "bscan_spi_xc7a420t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100",           loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("diffclk100",       loose=True), 1e9/100e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst.
    ("clk100", 0, Pins("P16"), IOStandard("LVCMOS25")),
    ("clk24",  0, Pins("M21"), IOStandard("LVCMOS25")),

    # Debug
    ("debug", 0, Pins("R7"), IOStandard("LVCMOS25")),
    ("debug", 1, Pins("R6"), IOStandard("LVCMOS25")),
    ("debug", 2, Pins("T7"), IOStandard("LVCMOS25")),
    ("debug", 3, Pins("T8"), IOStandard("LVCMOS25")),

    # Fan
    ("fan", 0, Pins("R18"), IOStandard("LVCMOS25")),

    # Flash
    ("flash_cs_n", 0, Pins("P18"), IOStandard("LVCMOS25")),
    ("flash", 0,
        Subsignal("mosi", Pins("R14")),
        Subsignal("miso", Pins("R15")),
        Subsignal("vpp",  Pins("P14")),
        Subsignal("hold", Pins("N14")),
        IOStandard("LVCMOS25")
    ),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("R7")), # debug0
        Subsignal("rx", Pins("R6")), # debug1
        IOStandard("LVCMOS25")
    ),

    # PCIe
    ("pcie_x4", 0,
        Subsignal("rst_n", Pins("M19"), IOStandard("LVCMOS25"), Misc("PULLUP=TRUE")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p",  Pins("B11 D14 B13 D12")),
        Subsignal("rx_n",  Pins("A11 C14 A13 C12")),
        Subsignal("tx_p",  Pins("B7   D8  B9 D10")),
        Subsignal("tx_n",  Pins("A7   C8  A9 C10"))
    ),

    # DDR3 SDRAM
    ("ddram", 0,
        Subsignal("a", Pins(
            "K2 M5 M2 K1 N6 J1 P1 H2",
            "R1 M1 M6 N3 M7 H1"),
            IOStandard("SSTL15")),
        Subsignal("ba",    Pins("K3 N2 L3"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("L7"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("L5"), IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("L2"), IOStandard("SSTL15")),
        Subsignal("cs_n",  Pins("K5"), IOStandard("SSTL15")),
        Subsignal("dm", Pins("G8 J6 D5 A3"), IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "G6 H8 F7 F8 D6 H9 E6 H6",
            "J5 G4 L8 F4 K6 G5 K7 K8",
            "A4 D4 B4 E5 C4 F3 C3 D3",
            "G1 D1 G2 A2 E1 E2 F2 C2"),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("H7 J4 B5 C1"),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_n", Pins("G7 H4 A5 B1"),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("clk_p", Pins("P4"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("N4"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke",   Pins("N7"), IOStandard("SSTL15")),
        Subsignal("odt",   Pins("J3"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("N1"), IOStandard("SSTL15")),
        Misc("SLEW=FAST"),
    ),

    # SDI
    ("sdi_refclk_sel", 0, Pins("AB26"), IOStandard("LVCMOS25")),
    ("sdi_refclk", 0,
        Subsignal("p", Pins("AA13")),
        Subsignal("n", Pins("AB13")),
    ),
    ("sdi_refclk", 1,
        Subsignal("p", Pins("AA11")),
        Subsignal("n", Pins("AB11")),
    ),
    ("sdi_data", 0,
        Subsignal("txp", Pins("AC10")),
        Subsignal("txn", Pins("AD10")),
        Subsignal("rxp", Pins("AC12")),
        Subsignal("rxn", Pins("AD12")),
    ),

    # HDMI (through 75DP159)
    ("hdmi_out", 0,
        Subsignal("clk_p",   Pins("U14"), IOStandard("LVDS_25")),
        Subsignal("clk_n",   Pins("V14"), IOStandard("LVDS_25")),
        Subsignal("data0_p", Pins("AE7")),
        Subsignal("data0_n", Pins("AF7")),
        Subsignal("data1_p", Pins("AC8")),
        Subsignal("data1_n", Pins("AD8")),
        Subsignal("data2_p", Pins("AE9")),
        Subsignal("data2_n", Pins("AF9")),
        # FIXME: Find a way to avoid RX pads.
        Subsignal("rx0_p", Pins("AE11")),
        Subsignal("rx0_n", Pins("AF11")),
        Subsignal("rx1_p", Pins("AC14")),
        Subsignal("rx1_n", Pins("AD14")),
        Subsignal("rx2_p", Pins("AE13")),
        Subsignal("rx2_n", Pins("AF13")),

    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a100t-fgg676-3", _io, toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 34]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 35]")

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft232.cfg", "bscan_spi_xc7a100t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("clk24",  loose=True), 1e9/24e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Icenowy Zheng <icenowy@aosc.io>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.build.generic_platform import *
from litex.build.gowin.platform import GowinPlatform
from litex.build.gowin.programmer import GowinProgrammer
from litex.build.openfpgaloader import OpenFPGALoader


# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk27",  0, Pins("52"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("10"), IOStandard("LVCMOS18")),
    ("user_led", 1, Pins("11"), IOStandard("LVCMOS18")),
    ("user_led", 2, Pins("13"), IOStandard("LVCMOS18")),
    ("user_led", 3, Pins("14"), IOStandard("LVCMOS18")),
    ("user_led", 4, Pins("15"), IOStandard("LVCMOS18")),
    ("user_led", 5, Pins("16"), IOStandard("LVCMOS18")),

    # Buttons.
    ("user_btn", 0, Pins("3"), IOStandard("LVCMOS18")),
    ("user_btn", 1, Pins("4"), IOStandard("LVCMOS18")),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("18")),
        Subsignal("tx", Pins("17")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("60"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("59"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("62"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("61"), IOStandard("LVCMOS33")),
    ),

    ("spisdcard", 0,
        Subsignal("clk",  Pins("36")),
        Subsignal("mosi", Pins("37")),
        Subsignal("cs_n", Pins("38")),
        Subsignal("miso", Pins("39")),
        IOStandard("LVCMOS33"),
    ),

    # PSRAM
    ("O_psram_ck",      0, Pins(2)),
    ("O_psram_ck_n",    0, Pins(2)),
    ("O_psram_cs_n",    0, Pins(2)),
    ("O_psram_reset_n", 0, Pins(2)),
    ("IO_psram_dq",     0, Pins(16)),
    ("IO_psram_rwds",   0, Pins(2)),

    # HDMI.
    ("hdmi", 0,
        Subsignal("clk_p",   Pins("69")),
        Subsignal("clk_n",   Pins("68")),
        Subsignal("data0_p", Pins("71")),
        Subsignal("data0_n", Pins("70")),
        Subsignal("data1_p", Pins("73")),
        Subsignal("data1_n", Pins("72")),
        Subsignal("data2_p", Pins("75")),
        Subsignal("data2_n", Pins("74")),
        Misc("PULL_MODE=NONE"),
    ),

    # SPI RGB LCD.
    ("spilcd", 0,
        Subsignal("reset", Pins("47")),
        Subsignal("cs",    Pins("48")),
        Subsignal("clk",   Pins("79")),
        Subsignal("mosi",  Pins("77")),
        Subsignal("rs",    Pins("47")),
        IOStandard("LVCMOS33"),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ["J6", "38 37 36 39 25 26 27 28 29 30 33 34 40 35 41 42 51 53 54 55 56 57 68 69"],
    ["J7", "63 86 85 84 83 82 81 80 79 77 76 75 74 73 72 71 70  - 48 49 31 32  -  -"],
]

# Platform -----------------------------------------------------------------------------------------

class Platform(GowinPlatform):
    default_clk_name   = "clk27"
    default_clk_period = 1e9/27e6

    def __init__(self, toolchain="gowin"):
        GowinPlatform.__init__(self, "GW1NR-LV9QN88PC6/I5", _io, _connectors, toolchain=toolchain, devicename="GW1NR-9C")
        self.toolchain.options["use_mspi_as_gpio"] = 1

    def create_programmer(self, kit="openfpgaloader"):
        if kit == "gowin":
            return GowinProgrammer(self.devicename)
        else: 
            return OpenFPGALoader(cable="ft2232")

    def do_finalize(self, fragment):
        GowinPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk27", loose=True), 1e9/27e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020-2021 Xuanyu Hu <xuanyu.hu@whu.edu.cn>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import digilent_basys3

from litex.soc.cores.clock import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.video import VideoVGAPHY
from litex.soc.cores.led import LedChaser

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        self.cd_vga = ClockDomain()

        self.pll = pll = S7MMCM(speedgrade=-1)
        self.comb += pll.reset.eq(platform.request("user_btnc") | self.rst)

        pll.register_clkin(platform.request("clk100"), 100e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)
        pll.create_clkout(self.cd_vga, 40e6)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.
        #platform.add_platform_command("set_property CLOCK_DEDICATED_ROUTE FALSE [get_nets clk100_IBUF]")

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=75e6, with_led_chaser=True, with_video_terminal=False, **kwargs):
        platform = digilent_basys3.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------_-----------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Basys3", **kwargs)

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal:
            self.videophy = VideoVGAPHY(platform.request("vga"), clock_domain="vga")
            if with_video_terminal:
                self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------  
def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=digilent_basys3.Platform, description="LiteX SoC on Basys3.")
    parser.add_target_argument("--sys-clk-freq", default=75e6, type=float, help="System clock frequency.")
    sdopts = parser.target_group.add_mutually_exclusive_group()
    sdopts.add_argument("--with-spi-sdcard",       action="store_true", help="Enable SPI-mode SDCard support.")
    sdopts.add_argument("--with-sdcard",           action="store_true", help="Enable SDCard support.")
    parser.add_target_argument("--sdcard-adapter",                      help="SDCard PMOD adapter (digilent or numato).")
    viopts = parser.target_group.add_mutually_exclusive_group()
    viopts.add_argument("--with-video-terminal", action="store_true", help="Enable Video Terminal (VGA).")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        with_video_terminal = args.with_video_terminal,
        **parser.soc_argdict
    )
    soc.platform.add_extension(digilent_basys3._sdcard_pmod_io)
    if args.with_spi_sdcard:
        soc.add_spi_sdcard()
    if args.with_sdcard:
        soc.add_sdcard()
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Piotr Esden-Tempski <piotr@esden.net>
# Copyright (c) 2021 Sylvain Munaut <tnt@246tNt.com>
# Copyright (c) 2023 Michael Welling <mwelling@ieee.org>
# SPDX-License-Identifier: BSD-2-Clause

# ICE-V wireless

from litex.build.dfu import DFUProg
from litex.build.generic_platform import *
from litex.build.lattice import LatticeiCE40Platform

# IOs ----------------------------------------------------------------------------------------------

_io_v0 = [
    # Clk / Rst
    ("clk12", 0, Pins("35"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led_n",    0, Pins("39"), IOStandard("LVCMOS33")),
    ("user_led_n",    1, Pins("40"), IOStandard("LVCMOS33")),
    ("user_led_n",    2, Pins("41"), IOStandard("LVCMOS33")),

    # Button
    ("user_btn_n",    0, Pins("19"), IOStandard("LVCMOS33")),

    # USB
    ("usb", 0,
        Subsignal("d_p", Pins("4")),
        Subsignal("d_n", Pins("3")),
        Subsignal("pullup", Pins("45")),
        IOStandard("LVCMOS33")
    ),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("11")),
        Subsignal("tx", Pins("12"), Misc("PULLUP")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("37"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("28"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("26"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("23"), IOStandard("LVCMOS33")),
        Subsignal("wp",   Pins("25"), IOStandard("LVCMOS33")),
        Subsignal("hold", Pins("27"), IOStandard("LVCMOS33")),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("37"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("28"), IOStandard("LVCMOS33")),
        Subsignal("dq",   Pins("26 23 25 27"), IOStandard("LVCMOS33")),
    ),
]


# Connectors ---------------------------------------------------------------------------------------

_connectors_v0 = [
    ("PMOD1", "19 12 11  9 18 21 10  8"),
    ("PMOD2", " 4 48 47 46  3 45 44  2"),
    ("PMOD3", "42 43 32 13 38 36 31 20")
]


# Platform -----------------------------------------------------------------------------------------

class Platform(LatticeiCE40Platform):
    default_clk_name   = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, revision="v0", toolchain="icestorm"):
        assert revision in ["v0"]
        io, connectors = {
            "v0": (_io_v0, _connectors_v0),
        }[revision]
        LatticeiCE40Platform.__init__(self, "ice40-up5k-sg48", io, connectors, toolchain=toolchain)

    def create_programmer(self):
        return DFUProg(vid="1d50", pid="6146")

    def do_finalize(self, fragment):
        LatticeiCE40Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk12", loose=True), 1e9/12e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2018-2019 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import kosagi_netv2

from litex.soc.interconnect.csr import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from litex.soc.cores.clock import *
from litex.soc.cores.led import LedChaser

from litedram.modules import K4B2G1646F
from litedram.phy import s7ddrphy

from liteeth.phy.rmii import LiteEthPHYRMII

from litepcie.phy.s7pciephy import S7PCIEPHY
from litepcie.software import generate_litepcie_software

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst          = Signal()
        self.cd_sys       = ClockDomain()
        self.cd_sys4x     = ClockDomain()
        self.cd_sys4x_dqs = ClockDomain()
        self.cd_idelay    = ClockDomain()
        self.cd_clk100    = ClockDomain()
        self.cd_eth       = ClockDomain()

        # Clk/Rst
        clk50 = platform.request("clk50")

        # PLL
        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(clk50, 50e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    200e6)
        pll.create_clkout(self.cd_clk100,    100e6)
        pll.create_clkout(self.cd_eth,       50e6)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, variant="a7-35", sys_clk_freq=100e6,
        with_pcie       = False,
        with_ethernet   = False,
        with_led_chaser = True,
        **kwargs):
        platform = kosagi_netv2.Platform(variant=variant)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on NeTV2", **kwargs)

        # DDR3 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = s7ddrphy.A7DDRPHY(platform.request("ddram"),
                memtype      = "DDR3",
                nphases      = 4,
                sys_clk_freq = sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = K4B2G1646F(sys_clk_freq, "1:4"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Ethernet ---------------------------------------------------------------------------------
        if with_ethernet:
            self.ethphy = LiteEthPHYRMII(
                clock_pads = self.platform.request("eth_clocks"),
                pads       = self.platform.request("eth"))
            self.add_ethernet(phy=self.ethphy)

        # PCIe -------------------------------------------------------------------------------------
        if with_pcie:
            self.pcie_phy = S7PCIEPHY(platform, platform.request("pcie_x4"),
                data_width = 128,
                bar0_size  = 0x20000)
            self.add_pcie(phy=self.pcie_phy, ndmas=1)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=kosagi_netv2.Platform, description="LiteX SoC on NeTV2.")
    parser.add_target_argument("--variant",       default="a7-35",           help="Board variant (a7-35 or a7-100).")
    parser.add_target_argument("--sys-clk-freq",  default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-ethernet", action="store_true",       help="Enable Ethernet support.")
    parser.add_target_argument("--with-pcie",     action="store_true",       help="Enable PCIe support.")
    parser.add_target_argument("--driver",        action="store_true",       help="Generate PCIe driver.")
    sdopts = parser.target_group.add_mutually_exclusive_group()
    sdopts.add_argument("--with-spi-sdcard", action="store_true", help="Enable SPI-mode SDCard support.")
    sdopts.add_argument("--with-sdcard",     action="store_true", help="Enable SDCard support.")
    args = parser.parse_args()

    soc = BaseSoC(
        variant       = args.variant,
        sys_clk_freq  = args.sys_clk_freq,
        with_ethernet = args.with_ethernet,
        with_pcie     = args.with_pcie,
        **parser.soc_argdict
    )
    if args.with_spi_sdcard:
        soc.add_spi_sdcard()
    if args.with_sdcard:
        soc.add_sdcard()
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.driver:
        generate_litepcie_software(soc, os.path.join(builder.output_dir, "driver"))

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Shinken Sanada <sanadashinken@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

# IOs specific to V1 of the board
_io_v1 = [
    # Reset (Key1 button)
    ("cpu_reset",  0, Pins("J8"),  IOStandard("LVCMOS33")),  # key1

    #Clock
    ("clk50"   ,   0, Pins("M22"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led",   0, Pins("J6"),   IOStandard("LVCMOS33")),
    ("user_led",   1, Pins("H6"),   IOStandard("LVCMOS33")),
]

# IOs specific to V2 of the board
_io_v2 = [
    # Reset (Key1 button)
    ("cpu_reset",  0, Pins("M6"),  IOStandard("LVCMOS33")),

    # Clock
    ("clk50"   ,   0, Pins("M21"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led",   0, Pins("V16"),   IOStandard("LVCMOS33")),
    ("user_led",   1, Pins("V17"),   IOStandard("LVCMOS33")),

    # SD-Card
    ("sdcard", 0,
     Subsignal("data", Pins("M5 M7 H6 J6")),
     Subsignal("cmd",  Pins("J8")),
     Subsignal("clk",  Pins("L4")),
     Subsignal("cd",   Pins("N6")),
     Misc("SLEW=FAST"),
     IOStandard("LVCMOS33"),
     ),
]

# IO commons to both versions of the board
_io_common = [
    # Key0 button (Key1 is used as cpu reset and is version specific)
    ("user_btn",   0, Pins("H7"),   IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("E3")),
        Subsignal("rx", Pins("F3")),
        IOStandard("LVCMOS33"),
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("P18")),
        Subsignal("clk",  Pins("H13")),
        Subsignal("mosi", Pins("R14")),
        Subsignal("miso", Pins("R15")),
        Subsignal("wp",   Pins("P14")),
        Subsignal("hold", Pins("N14")),
        IOStandard("LVCMOS33"),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("P18")),
        Subsignal("clk",  Pins("H13")),
        Subsignal("dq",   Pins("R14 R15 P14 N14")),
        IOStandard("LVCMOS33")
    ),

    # DDR3 SDRAM
    ("ddram", 0,
        Subsignal("a", Pins(
            "E17 G17 F17 C17 G16 D16 H16 E16",
            "H14 F15 F20 H15 C18 G15"),
            IOStandard("SSTL135")),
        Subsignal("ba",    Pins("B17 D18 A17"), IOStandard("SSTL135")),
        Subsignal("ras_n", Pins("A19"),  IOStandard("SSTL135")),
        Subsignal("cas_n", Pins("B19"),  IOStandard("SSTL135")),
        Subsignal("we_n",  Pins("A18"),  IOStandard("SSTL135")),
        # cs_n is only wired on V1 of the board but E22 is unconnected on V2
        # so leaving this here shouldn't hurt
        Subsignal("cs_n",  Pins("E22"),  IOStandard("SSTL135")),
        Subsignal("dm", Pins("A22 C22"), IOStandard("SSTL135")),
        Subsignal("dq", Pins(
            "D21 C21 B22 B21 D19 E20 C19 D20",
            "C23 D23 B24 B25 C24 C26 A25 B26"),
            IOStandard("SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("B20 A23"),
            IOStandard("DIFF_SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_n", Pins("A20 A24"),
            IOStandard("DIFF_SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("clk_p", Pins("F18"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_n", Pins("F19"), IOStandard("DIFF_SSTL135")),
        Subsignal("cke",   Pins("E18"), IOStandard("SSTL135")),
        Subsignal("odt",   Pins("G19"), IOStandard("SSTL135")),
        Subsignal("reset_n", Pins("H17"), IOStandard("SSTL135")),
        Misc("SLEW=FAST"),
    ),

    # GMII Ethernet
    ("eth_clocks", 0,
        Subsignal("tx", Pins("M2")),
        Subsignal("gtx", Pins("U1")),
        Subsignal("rx", Pins("P4")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("R1")),
        Subsignal("mdio",    Pins("H1")),
        Subsignal("mdc",     Pins("H2")),
        Subsignal("rx_dv",   Pins("L3")),
        Subsignal("rx_er",   Pins("U5")),
        Subsignal("rx_data", Pins("M4 N3 N4 P3 R3 T3 T4 T5")),
        Subsignal("tx_en",   Pins("T2")),
        Subsignal("tx_er",   Pins("J1")),
        Subsignal("tx_data", Pins("R2 P1 N2 N1 M1 L2 K2 K1")),
        Subsignal("col",     Pins("U4")),
        Subsignal("crs",     Pins("U2")),
        IOStandard("LVCMOS33")
    ),

    # HDMI out
    ("hdmi_out", 0,
        Subsignal("clk_p",   Pins("D4"), IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("C4"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("E1"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("D1"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("F2"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("E2"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("G2"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("G1"), IOStandard("TMDS_33")),
        Subsignal("scl",     Pins("B2"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("A2"), IOStandard("LVCMOS33")),
        Subsignal("hdp",     Pins("A3"), IOStandard("LVCMOS33")),
        Subsignal("cec",     Pins("B1"), IOStandard("LVCMOS33")),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("j10", "D5 G5 G7 G8 E5 E6 D6 G6"),
    ("j11", "H4 F4 A4 A5 J4 G4 B4 B5"),
    ("j12", "AB26 AC26 AB24 AC24 AA24 AB25 AA22 AA23",
            " Y25 AA25  W25  Y26  Y22  Y23  W21  Y21",
            " V26  W26  U25  U26  V24  W24  V23  W23",
            " V18  W18  U22  V22  U21  V21  T20  U20",
            " T19 U19"),
    ("jp2", "H21 H22 K21 J21 H26 G26 G25 F25",
            "G20 G21 F23 E23 E26 D26 E25 D25"),
    ("jp3", " AF7  AE7  AD8  AC8  AF9  AE9 AD10 AC10",
            "AA11 AB11 AF11 AE11 AD14 AC14 AF13 AE13",
            "AD12 AC12"),
]

# PMODS --------------------------------------------------------------------------------------------

def sdcard_pmod_io(pmod):
    return [
        # SDCard PMOD:
        # - https://store.digilentinc.com/pmod-microsd-microsd-card-slot/
        ("spisdcard", 0,
            Subsignal("clk",  Pins(f"{pmod}:3")),
            Subsignal("mosi", Pins(f"{pmod}:1"), Misc("PULLUP True")),
            Subsignal("cs_n", Pins(f"{pmod}:0"), Misc("PULLUP True")),
            Subsignal("miso", Pins(f"{pmod}:2"), Misc("PULLUP True")),
            Misc("SLEW=FAST"),
            IOStandard("LVCMOS33"),
        ),
        ("sdcard", 0,
            Subsignal("data", Pins(f"{pmod}:2 {pmod}:4 {pmod}:5 {pmod}:0"), Misc("PULLUP True")),
            Subsignal("cmd",  Pins(f"{pmod}:1"), Misc("PULLUP True")),
            Subsignal("clk",  Pins(f"{pmod}:3")),
            Subsignal("cd",   Pins(f"{pmod}:6")),
            Misc("SLEW=FAST"),
            IOStandard("LVCMOS33"),
        ),
]
_sdcard_pmod_io = sdcard_pmod_io("j10") # SDCARD PMOD on J10.
def ps2_pmod_io(pmod):
    return [
        ("ps2kbd", 0, Pins(f"{pmod}:0 {pmod}:2"), IOStandard("LVCMOS33")), # data, clk
]
_ps2_pmod_io = ps2_pmod_io("j11") # PS2 PMOD on top line of J11

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, board_version=1, speed_grade=-2, toolchain="vivado"):
        io = _io_common
        if board_version < 2:
            io.extend(_io_v1)
        else:
            io.extend(_io_v2)
        Xilinx7SeriesPlatform.__init__(self, "xc7a100t{}fgg676".format(speed_grade), io, _connectors,  toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.675 [get_iobanks 16]")
        if board_version < 2:
            self.add_platform_command("set_property CLOCK_DEDICATED_ROUTE FALSE [get_nets clk50_IBUF]")
        self.add_platform_command("set_property CFGBVS VCCO [current_design]")
        self.add_platform_command("set_property CONFIG_VOLTAGE 3.3 [current_design]")

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft232.cfg", "bscan_spi_xc7a100t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Do Viet Thanh <thanhdv@soc.one>
# Copyright (c) 2020 Fei Gao <feig@princeton.edu>
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2020 David Shah <dave@ds0.me>
# SPDX-License-Identifier: BSD-2-Clause

import os

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import xilinx_alveo_u200

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from litex.soc.cores.led import LedChaser
from litedram.modules import MTA18ASF2G72PZ
from litedram.phy import usddrphy

from litepcie.phy.usppciephy import USPPCIEPHY
from litepcie.software import generate_litepcie_software

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst       = Signal()
        self.cd_sys    = ClockDomain()
        self.cd_sys4x  = ClockDomain()
        self.cd_pll4x  = ClockDomain()
        self.cd_idelay = ClockDomain()

        # # #

        self.pll = pll = USMMCM(speedgrade=-2)
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(platform.request("clk300", 0), 300e6)
        pll.create_clkout(self.cd_pll4x, sys_clk_freq*4, buf=None, with_reset=False)
        pll.create_clkout(self.cd_idelay, 500e6)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.specials += [
            Instance("BUFGCE_DIV",
                p_BUFGCE_DIVIDE=4,
                i_CE=1, i_I=self.cd_pll4x.clk, o_O=self.cd_sys.clk),
            Instance("BUFGCE",
                i_CE=1, i_I=self.cd_pll4x.clk, o_O=self.cd_sys4x.clk),
        ]

        self.idelayctrl = USIDELAYCTRL(cd_ref=self.cd_idelay, cd_sys=self.cd_sys)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=125e6, with_led_chaser=True, with_pcie=False, **kwargs):
        platform = xilinx_alveo_u200.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Alveo U200", **kwargs)

        # DDR4 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = usddrphy.USPDDRPHY(platform.request("ddram"),
                memtype          = "DDR4",
                sys_clk_freq     = sys_clk_freq,
                cmd_latency      = 1,
                iodelay_clk_freq = 500e6,
                is_rdimm         = True)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MTA18ASF2G72PZ(sys_clk_freq, "1:4"),
                size          = 0x40000000,
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Firmware RAM (To ease initial LiteDRAM calibration support) ------------------------------
        self.add_ram("firmware_ram", 0x20000000, 0x8000)

        # PCIe -------------------------------------------------------------------------------------
        if with_pcie:
            self.pcie_phy = USPPCIEPHY(platform, platform.request("pcie_x4"),
                data_width = 128,
                bar0_size  = 0x20000)
            self.add_pcie(phy=self.pcie_phy, ndmas=1)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=xilinx_alveo_u200.Platform, description="LiteX SoC on Alveo U200.")
    parser.add_target_argument("--sys-clk-freq", default=125e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-pcie",    action="store_true",       help="Enable PCIe support.")
    parser.add_target_argument("--driver",       action="store_true",       help="Generate PCIe driver.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        with_pcie    = args.with_pcie,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.driver:
        generate_litepcie_software(soc, os.path.join(builder.output_dir, "driver"))

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Rakesh Peter <rakesh@stanproc.in>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform, VivadoProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("sysclk", 0, Pins("H16"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("R14"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("P14"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("N16"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("M14"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("L15"), IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("G17"), IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("N15"), IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("G14"), IOStandard("LVCMOS33")),
    ("user_led", 8, Pins("L14"), IOStandard("LVCMOS33")),
    ("user_led", 9, Pins("M15"), IOStandard("LVCMOS33")),
  
    # Switches
    ("user_sw", 0, Pins("M20"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("M19"), IOStandard("LVCMOS33")),
    
    # Buttons
    ("user_btn", 0, Pins("D19"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("D20"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("L20"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("L19"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("pmoda:0")),
        Subsignal("rx", Pins("pmoda:1")),
        IOStandard("LVCMOS33")
    ),

    # Audio
    ("aud_pwm", 0, Pins("R18"), IOStandard("LVCMOS33")),
    ("aud_sd",  0, Pins("T17"), IOStandard("LVCMOS33")),
    ("m_clk",   0, Pins("F17"), IOStandard("LVCMOS33")),
    ("m_data",  0, Pins("G18"), IOStandard("LVCMOS33")),

    # Chipkit Single Ended Analog Input
    ("ck_an_n", 0, Pins("D18"), IOStandard("LVCMOS33")),
    ("ck_an_p", 0, Pins("E17"), IOStandard("LVCMOS33")),
    ("ck_an_n", 1, Pins("E19"), IOStandard("LVCMOS33")),
    ("ck_an_p", 1, Pins("E18"), IOStandard("LVCMOS33")),
    ("ck_an_n", 2, Pins("J14"), IOStandard("LVCMOS33")),
    ("ck_an_p", 2, Pins("K14"), IOStandard("LVCMOS33")),
    ("ck_an_n", 3, Pins("J16"), IOStandard("LVCMOS33")),
    ("ck_an_p", 3, Pins("K16"), IOStandard("LVCMOS33")),
    ("ck_an_n", 4, Pins("H20"), IOStandard("LVCMOS33")),
    ("ck_an_p", 4, Pins("J20"), IOStandard("LVCMOS33")),
    ("ck_an_n", 5, Pins("G20"), IOStandard("LVCMOS33")),
    ("ck_an_p", 5, Pins("G19"), IOStandard("LVCMOS33")),

    # Chipkit SPI
    ("ck_miso", 0, Pins("W15"), IOStandard("LVCMOS33")),
    ("ck_mosi", 0, Pins("T12"), IOStandard("LVCMOS33")),
    ("ck_sck",  0, Pins("H15"), IOStandard("LVCMOS33")),
    ("ck_ss",   0, Pins("F16"), IOStandard("LVCMOS33")),
    
    # Chipkit I2C
    ("ck_scl", 0, Pins("P16"), IOStandard("LVCMOS33")),
    ("ck_sda", 0, Pins("P15"), IOStandard("LVCMOS33")),

    # Crypto SDA
    ("crypto_sda", 0, Pins("J15"), IOStandard("LVCMOS33")),
]

_ps7_io = [    
    # PS7
    ("ps7_clk",   0, Pins("E7")),
    ("ps7_porb",  0, Pins("C7")),
    ("ps7_srstb", 0, Pins("B10")),
    ("ps7_mio",   0, Pins(
        " E6  A7  B8  D6  B7  A6  A5  D8",
        " D5  B5  E9  C6  D9  E8  C5  C8",
        "A19 E14 B18 D10 A17 F14 B17 D11",
        "A16 F15 A15 D13 C16 C13 C15 E16",
        "A14 D15 A12 F12 A11 A10 E13 C18"
        "D14 C17 E12  A9 F13 B15 D16 B14"
        "B12 C12 B13  B9 C10 C11")),
    ("ps7_ddram", 0,
        Subsignal("addr", Pins(
            "N2 K2 M3 K3 M4 L1 L4 K4",
            "K1 J4 F5 G4 E4 D4 F4")),
        Subsignal("ba",    Pins("L5 R4 J5")),
        Subsignal("cas_n", Pins("P5")),
        Subsignal("cke",   Pins("N3")),
        Subsignal("ck_n",  Pins("M2")),
        Subsignal("ck_p",  Pins("L2")),
        Subsignal("cs_n",  Pins("N1")),
        Subsignal("dm",    Pins("A1 F1 T1 Y1")),
        Subsignal("dq", Pins(
            "C3 B3 A2 A4 D3 D1 C1 E1",
            "E2 E3 G3 H3 J3 H2 H1 J1",
            "P1 P3 R3 R1 T4 U4 U2 U3",
            "V1 Y3 W1 Y4 Y2 W3 V2 V3"),
        ),
        Subsignal("dqs_n",   Pins("B2 F2 T2 W4")),
        Subsignal("dqs_p",   Pins("C2 G2 R2 W5")),
        Subsignal("reset_n", Pins("B4")),
        Subsignal("odt",     Pins("N5")),
        Subsignal("ras_n",   Pins("P4")),
        Subsignal("vrn",     Pins("G5")),
        Subsignal("vrp",     Pins("H5")),
        Subsignal("we_n",    Pins("M5"))
    ),
]

_hdmi_rx_io = [
    # HDMI Rx
    ("hdmi_rx", 0,
        Subsignal("cec",     Pins("H17"), IOStandard("LVCMOS33")),
        Subsignal("clk_p",   Pins("N18"), IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("P19"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("V20"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("W20"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("T20"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("U20"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("N20"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("P20"), IOStandard("TMDS_33")),
        Subsignal("hpd",     Pins("T19"), IOStandard("LVCMOS33")),
        Subsignal("scl",     Pins("U14"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("U15"), IOStandard("LVCMOS33")),
 ),
]

_hdmi_tx_io = [
    # HDMI Tx
    ("hdmi_tx", 0,
        Subsignal("cec",     Pins("G15"), IOStandard("LVCMOS33")),
        Subsignal("clk_p",   Pins("L16"), IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("L17"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("K17"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("K18"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("K19"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("J19"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("J18"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("H18"), IOStandard("TMDS_33")),
        Subsignal("hpdn",    Pins("R19"), IOStandard("LVCMOS33")),
        Subsignal("scl",     Pins("M17"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("M18"), IOStandard("LVCMOS33")),
 ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("pmoda", "Y18 Y19 Y16 Y17 U18 U19 W18 W19"), # XADC
    ("pmodb", "W14 Y14 T11 T10 V16 W16 V12 W13"),
    ("ck_io", "T14 U12 U13 V13 V15 T15 R16 U17 V17 V18 T16 R17 P18 N17 Y11 Y12 W11 V11 T5 U10 B20 C20 F20 F19 A20 B19 U5 V5 V6 U7 V7 U8 V8 V10 W10 W6 Y6 Y7 W8 Y8 W9 Y9 Y13")
]
# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "sysclk"
    default_clk_period = 1e9/125e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7z020-clg400-1", _io,  _connectors, toolchain=toolchain)
        self.add_extension(_ps7_io)
        self.add_extension(_hdmi_rx_io)
        self.add_extension(_hdmi_tx_io)

    def create_programmer(self):
        return VivadoProgrammer()

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("sysclk", loose=True), 1e9/125e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform, VivadoProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # GPIOs
    ("gpio", 0, Pins("K13"), IOStandard("LVCMOS18")),
    ("gpio", 1, Pins("M12"), IOStandard("LVCMOS18")),
    ("gpio", 2, Pins("R10"), IOStandard("LVCMOS18")),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = []

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7z010clg225-1", _io,  _connectors, toolchain=toolchain)

    def create_programmer(self):
        return VivadoProgrammer()

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Vamsi K Vytla <vamsi.vytla@gmail.com>
# Copyright (c) 2021 Michael Betz <michibetz@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause
#
# Marble is a dual FMC FPGA carrier board developed for general purpose use in
# particle accelerator electronics instrumentation. It is currently under
# development and the base platform for two accelerator projects at DOE: ALS-U
# (the Advanced Light Source Upgrade at LBNL and the LCLS-II HE (the Linac
# Coherent Light Source II High Energy upgrade).
# https://github.com/BerkeleyLab/Marble
#
# Generated by gen_marble.py (git 4a42959)
# https://github.com/yetifrisstlama/litex_test_project/blob/4a42959/xdc/gen_marble.py
# Pin numbers extracted from a .xdc file, which was auto-generated from
# the Kicad Schematic.

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    ("eth", 0,
        Subsignal("rst_n", Pins("B9"), IOStandard("LVCMOS25")),
        Subsignal("rx_ctl", Pins("J11"), IOStandard("LVCMOS25")),
        Subsignal("rx_data", Pins("J10 J8 H8 H9"), IOStandard("LVCMOS25")),
        Subsignal("tx_ctl", Pins("C9"), IOStandard("LVCMOS25")),
        Subsignal("tx_data", Pins("H11 H12 D8 D9"), IOStandard("LVCMOS25")),
    ),
    ("eth_clocks", 0,
        Subsignal("tx", Pins("F10"), IOStandard("LVCMOS25")),
        Subsignal("rx", Pins("E11"), IOStandard("LVCMOS25")),
    ),
    # Tunable VCXO. Warning: Non clock-capable pin
    ("clk20", 0, Pins("W11"), IOStandard("LVCMOS15")),
    # Main system clock. White rabbit compatible
    ("clk125", 0,
        Subsignal("p", Pins("AC9"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("AD9"), IOStandard("DIFF_SSTL15")),
    ),
    # 4x Multi gigabit clocks from cross-point switch, source configured by MMC
    ("clkmgt", 0,
        Subsignal("p", Pins("D6"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("D5"), IOStandard("DIFF_SSTL15")),
    ),
    ("clkmgt", 1,
        Subsignal("p", Pins("F6"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("F5"), IOStandard("DIFF_SSTL15")),
    ),
    ("clkmgt", 2,
        Subsignal("p", Pins("H6"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("H5"), IOStandard("DIFF_SSTL15")),
    ),
    ("clkmgt", 3,
        Subsignal("p", Pins("K6"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("K5"), IOStandard("DIFF_SSTL15")),
    ),
    # 2x LED: LD16 and LD17
    ("user_led", 0, Pins("Y13"), IOStandard("LVCMOS15")),
    ("user_led", 1, Pins("V12"), IOStandard("LVCMOS15")),
    # USB UART
    ("serial", 0,
        Subsignal("tx", Pins("K15"), IOStandard("LVCMOS25")),
        Subsignal("rts", Pins("M16"), IOStandard("LVCMOS25")),
        Subsignal("rx", Pins("C16"), IOStandard("LVCMOS25")),
    ),
    # I2C system bus, shared access with microcontroller
    # connected to TCA9548A I2C-multiplexer
    ("i2c_fpga", 0,
        Subsignal("scl", Pins("B16"), IOStandard("LVCMOS25")),
        Subsignal("sda", Pins("A17"), IOStandard("LVCMOS25")),
        Subsignal("rst", Pins("B19"), IOStandard("LVCMOS25")),
    ),
    # QSPI Boot Flash
    # access clock via STARTUPE2 primitive, wp_n may not be connected.
    ("spiflash", 0,
        Subsignal("cs_n", Pins("C23"), IOStandard("LVCMOS25")),
        Subsignal("mosi", Pins("B24"), IOStandard("LVCMOS25")),
        Subsignal("miso", Pins("A25"), IOStandard("LVCMOS25")),
        Subsignal("wp_n", Pins("B22"), IOStandard("LVCMOS25")),
    ),
    # 2x DAC for white rabbit frequency control
    ("wr_dac", 0,
        Subsignal("clk", Pins("V11"), IOStandard("LVCMOS15")),
        Subsignal("din", Pins("Y10"), IOStandard("LVCMOS15")),
        Subsignal("synca", Pins("W10"), IOStandard("LVCMOS15")),
        Subsignal("syncb", Pins("Y11"), IOStandard("LVCMOS15")),
    ),
    # DDR3 module
    ("ddram", 0,
        Subsignal("a", Pins("AC8 AB10 AA9 AA10 AD10 AC12 AB11 AC11 AF13 AE13 AE10 AD11 AA12 AE8 AB12 AD13"), IOStandard("SSTL15")),
        Subsignal("ba", Pins("AF10 AD8 AC13"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("AB7"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("AF8"), IOStandard("SSTL15")),
        Subsignal("we_n", Pins("AF9"), IOStandard("SSTL15")),
        Subsignal("cs_n", Pins("AC7"), IOStandard("SSTL15")),
        Subsignal("dm", Pins("AF17 W15 AC19 AA15 AC3 AD4 W1 U7"), IOStandard("SSTL15")),
        Subsignal("dq", Pins("AF20 AF19 AE17 AE15 AD16 AD15 AF15 AF14 V17 Y17 V18 V19 V16 W16 V14 W14 AA20 AD19 AB17 AC17 AA19 AB19 AD18 AC18 AA18 AB16 AA14 AD14 AB15 AA17 AC14 AB14 AD6 AB6 Y6 AC4 AC6 AB4 AA4 Y5 AF2 AE2 AE1 AD1 AE5 AE6 AF3 AE3 AA3 AC2 V2 V1 AB2 Y3 Y2 Y1 W3 V4 U2 U1 V6 V3 U6 U5"), IOStandard("SSTL15")),
        Subsignal("dqs_p", Pins("AE18 W18 AD20 Y15 AA5 AF5 AB1 W6"), IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n", Pins("AF18 W19 AE20 Y16 AB5 AF4 AC1 W5"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p", Pins("AE12"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("AF12"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke", Pins("AA13"), IOStandard("SSTL15")),
        Subsignal("odt", Pins("AB9"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("Y12"), IOStandard("SSTL15")),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("fmca", {
        "CLK0_M2C_N": "E17",
        "CLK0_M2C_P": "F17",
        "CLK1_M2C_N": "D18",
        "CLK1_M2C_P": "E18",
        "LA0_N": "H18",
        "LA0_P": "H17",
        "LA1_N": "F18",
        "LA1_P": "G17",
        "LA2_N": "J20",
        "LA2_P": "K20",
        "LA3_N": "L18",
        "LA3_P": "M17",
        "LA4_N": "G20",
        "LA4_P": "H19",
        "LA5_N": "E20",
        "LA5_P": "F19",
        "LA6_N": "L20",
        "LA6_P": "L19",
        "LA7_N": "D20",
        "LA7_P": "D19",
        "LA8_N": "F20",
        "LA8_P": "G19",
        "LA9_N": "J19",
        "LA9_P": "J18",
        "LA10_N": "G16",
        "LA10_P": "H16",
        "LA11_N": "K18",
        "LA11_P": "L17",
        "LA12_N": "F15",
        "LA12_P": "G15",
        "LA13_N": "D16",
        "LA13_P": "D15",
        "LA14_N": "E16",
        "LA14_P": "E15",
        "LA15_N": "J16",
        "LA15_P": "J15",
        "LA16_N": "K17",
        "LA16_P": "K16",
        "LA17_N": "D10",
        "LA17_P": "E10",
        "LA18_N": "C11",
        "LA18_P": "C12",
        "LA19_N": "G14",
        "LA19_P": "H14",
        "LA20_N": "A15",
        "LA20_P": "B15",
        "LA21_N": "D13",
        "LA21_P": "D14",
        "LA22_N": "A14",
        "LA22_P": "B14",
        "LA23_N": "F12",
        "LA23_P": "G12",
        "LA24_N": "A8",
        "LA24_P": "A9",
        "LA25_N": "G9",
        "LA25_P": "G10",
        "LA26_N": "E12",
        "LA26_P": "E13",
        "LA27_N": "F13",
        "LA27_P": "F14",
        "LA28_N": "H13",
        "LA28_P": "J13",
        "LA29_N": "F8",
        "LA29_P": "F9",
        "LA30_N": "B11",
        "LA30_P": "B12",
        "LA31_N": "A12",
        "LA31_P": "A13",
        "LA32_N": "C13",
        "LA32_P": "C14",
        "LA33_N": "A10",
        "LA33_P": "B10",
    }),
    ("fmcb", {
        "CLK0_M2C_N": "AA24",
        "CLK0_M2C_P": "Y23",
        "CLK1_M2C_N": "E23",
        "CLK1_M2C_P": "F22",
        "HA00_CC_N": "N22",
        "HA00_CC_P": "N21",
        "HA01_CC_N": "P21",
        "HA01_CC_P": "R21",
        "HA02_N": "U20",
        "HA02_P": "U19",
        "HA03_N": "T19",
        "HA03_P": "T18",
        "HA04_N": "R17",
        "HA04_P": "R16",
        "HA05_N": "N17",
        "HA05_P": "P16",
        "HA06_N": "P18",
        "HA06_P": "R18",
        "HA07_N": "T25",
        "HA07_P": "T24",
        "HA08_N": "T23",
        "HA08_P": "T22",
        "HA09_N": "T17",
        "HA09_P": "U17",
        "HA10_N": "K26",
        "HA10_P": "K25",
        "HA11_N": "M19",
        "HA11_P": "N18",
        "HA12_N": "L24",
        "HA12_P": "M24",
        "HA13_N": "R20",
        "HA13_P": "T20",
        "HA14_N": "P20",
        "HA14_P": "P19",
        "HA15_N": "P25",
        "HA15_P": "R25",
        "HA16_N": "P26",
        "HA16_P": "R26",
        "HA17_CC_N": "R23",
        "HA17_CC_P": "R22",
        "HA18_N": "M22",
        "HA18_P": "M21",
        "HA19_N": "M20",
        "HA19_P": "N19",
        "HA20_N": "N23",
        "HA20_P": "P23",
        "HA21_N": "N24",
        "HA21_P": "P24",
        "HA22_N": "M26",
        "HA22_P": "N26",
        "HA23_N": "L25",
        "HA23_P": "M25",
        "LA0_N": "AA22",
        "LA0_P": "Y22",
        "LA1_N": "AB24",
        "LA1_P": "AA23",
        "LA2_N": "AF22",
        "LA2_P": "AE22",
        "LA3_N": "AE26",
        "LA3_P": "AD26",
        "LA4_N": "W21",
        "LA4_P": "V21",
        "LA5_N": "AC26",
        "LA5_P": "AB26",
        "LA6_N": "AD24",
        "LA6_P": "AD23",
        "LA7_N": "AC22",
        "LA7_P": "AB22",
        "LA8_N": "AC24",
        "LA8_P": "AC23",
        "LA9_N": "V26",
        "LA9_P": "U26",
        "LA10_N": "AF23",
        "LA10_P": "AE23",
        "LA11_N": "W24",
        "LA11_P": "W23",
        "LA12_N": "AB25",
        "LA12_P": "AA25",
        "LA13_N": "V24",
        "LA13_P": "V23",
        "LA14_N": "U25",
        "LA14_P": "U24",
        "LA15_N": "V22",
        "LA15_P": "U22",
        "LA16_N": "W26",
        "LA16_P": "W25",
        "LA17_N": "F23",
        "LA17_P": "G22",
        "LA18_N": "F24",
        "LA18_P": "G24",
        "LA19_N": "J23",
        "LA19_P": "K23",
        "LA20_N": "K22",
        "LA20_P": "L22",
        "LA21_N": "H22",
        "LA21_P": "J21",
        "LA22_N": "D25",
        "LA22_P": "E25",
        "LA23_N": "H24",
        "LA23_P": "H23",
        "LA24_N": "J25",
        "LA24_P": "J24",
        "LA25_N": "D24",
        "LA25_P": "D23",
        "LA26_N": "E26",
        "LA26_P": "F25",
        "LA27_N": "G21",
        "LA27_P": "H21",
        "LA28_N": "G26",
        "LA28_P": "G25",
        "LA29_N": "H26",
        "LA29_P": "J26",
        "LA30_N": "C26",
        "LA30_P": "D26",
        "LA31_N": "E22",
        "LA31_P": "E21",
        "LA32_N": "A20",
        "LA32_P": "B20",
        "LA33_N": "B21",
        "LA33_P": "C21",
    }),
    ("pmoda", "C24 C22 L23 D21 K21 C18 C19 C17"),
    ("pmodb", "AE7 V7 Y7 AF7 V8 AA8 Y8 W9"),
]


# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk125"
    default_clk_period = 1e9 / 125e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7k160t-ffg676-2", _io, _connectors, toolchain=toolchain)
        self.toolchain.bitstream_commands = [
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"
        ]
        self.toolchain.additional_commands = [
            "write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"
        ]

        # from pin_map.csv: This is a frequency source, not a phase source, so having it enter on a non-CC pin is OK.
        self.add_platform_command("set_property CLOCK_DEDICATED_ROUTE FALSE [get_nets clk20_IBUF]")
        self.add_platform_command("set_property CONFIG_VOLTAGE 2.5 [current_design]")
        self.add_platform_command("set_property CFGBVS VCCO [current_design]")

        # TODO
        # self.add_platform_command("set_property INTERNAL_VREF 0.675 [get_iobanks 35]")

    def create_programmer(self):
        # same file works for marble mini and for marble
        return OpenOCD("openocd_marblemini.cfg")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk20", loose=True), 1e9/20e6)
        self.add_period_constraint(self.lookup_request("clk125", loose=True), 1e9/125e6)



#!/usr/bin/env python3
#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Kazumoto Kojima
# Copyright (c) 2023 Hans Baier <hansfbaier@gmail.com>
#
# SPDX-License-Identifier: BSD-2-Clause

import os
import argparse

from migen import *

from litex_boards.platforms import qmtech_xc7k325t
from litex.build.xilinx.vivado import vivado_build_args, vivado_build_argdict

from litex.soc.cores.clock import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.video import VideoVGAPHY
from litex.soc.cores.led import LedChaser

from litedram.modules import MT41J128M16
from litedram.phy import s7ddrphy

from liteeth.phy.mii import LiteEthPHYMII

# CRG ----------------------------------------------------------------------------------------------

class _CRG(Module):
    def __init__(self, platform, sys_clk_freq, with_ethernet, with_vga):
        self.rst = Signal()
        self.clock_domains.cd_sys       = ClockDomain()
        self.clock_domains.cd_sys4x     = ClockDomain(reset_less=True)
        self.clock_domains.cd_sys4x_dqs = ClockDomain(reset_less=True)
        self.clock_domains.cd_idelay    = ClockDomain()
        self.clock_domains.cd_eth       = ClockDomain()
        if with_ethernet:
            self.clock_domains.cd_eth   = ClockDomain()
        if with_vga:
            self.clock_domains.cd_vga   = ClockDomain(reset_less=True)

        # # #

        self.submodules.pll = pll = S7PLL(speedgrade=-1)
        reset_button = platform.request("cpu_reset")
        self.comb += pll.reset.eq(~reset_button | self.rst)

        pll.register_clkin(platform.request("clk50"), 50e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    200e6)
        if with_ethernet:
            pll.create_clkout(self.cd_eth,   25e6)
        if with_vga:
            pll.create_clkout(self.cd_vga,   40e6)

        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.submodules.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, toolchain="vivado", sys_clk_freq=int(100e6), with_daughterboard=False,
                 with_ethernet=False, with_etherbone=False, eth_ip="192.168.1.50", eth_dynamic_ip=False,
                 local_ip="", remote_ip="",
                 with_led_chaser=True, with_video_terminal=False, with_video_framebuffer=False, with_video_colorbars=False,
                 with_jtagbone=True, with_spi_flash=False, **kwargs):
        platform = qmtech_xc7k325t.Platform(toolchain=toolchain, with_daughterboard=with_daughterboard)

        # SoCCore ----------------------------------------------------------------------------------
        print(f"{str(kwargs)}")
        if (kwargs["uart_name"] == "serial") and (not with_daughterboard):
            kwargs["uart_name"] = "gpio_serial"

        SoCCore.__init__(self, platform, sys_clk_freq,
            ident = "LiteX SoC on QMTech XC7K325T" + (" + Daughterboard" if with_daughterboard else ""),
            **kwargs)

        # CRG --------------------------------------------------------------------------------------
        self.submodules.crg = _CRG(platform, sys_clk_freq, with_ethernet or with_etherbone, with_video_terminal or with_video_framebuffer or with_video_colorbars)

        # DDR3 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            from litedram.common import PHYPadsReducer
            self.submodules.ddrphy = s7ddrphy.A7DDRPHY(
                pads           = PHYPadsReducer(platform.request("ddram", 0), [0, 1]),
                memtype        = "DDR3",
                nphases        = 4,
                sys_clk_freq   = sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MT41J128M16(sys_clk_freq, "1:4"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            self.submodules.ethphy = LiteEthPHYMII(
                clock_pads = self.platform.request("eth_clocks"),
                pads       = self.platform.request("eth"))
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy, dynamic_ip=eth_dynamic_ip)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy, ip_address=eth_ip)
            # The daughterboard has the tx clock wired to a non-clock pin, so we can't help it
            #self.platform.add_platform_command("set_property CLOCK_DEDICATED_ROUTE FALSE [get_nets eth_clocks_tx_IBUF]")
            self.add_constant("TARGET_BIOS_INIT", 1)

        if local_ip:
            local_ip = local_ip.split(".")
            self.add_constant("LOCALIP1", int(local_ip[0]))
            self.add_constant("LOCALIP2", int(local_ip[1]))
            self.add_constant("LOCALIP3", int(local_ip[2]))
            self.add_constant("LOCALIP4", int(local_ip[3]))

        if remote_ip:
            remote_ip = remote_ip.split(".")
            self.add_constant("REMOTEIP1", int(remote_ip[0]))
            self.add_constant("REMOTEIP2", int(remote_ip[1]))
            self.add_constant("REMOTEIP3", int(remote_ip[2]))
            self.add_constant("REMOTEIP4", int(remote_ip[3]))

        # Jtagbone ---------------------------------------------------------------------------------
        if with_jtagbone:
            self.add_jtagbone()

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            from litespi.modules import MT25QL128
            from litespi.opcodes import SpiNorFlashOpCodes as Codes
            self.add_spi_flash(mode="4x", module=MT25QL128(Codes.READ_1_1_1), with_master=True)

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal or with_video_framebuffer or with_video_colorbars:
            self.submodules.videophy = VideoVGAPHY(platform.request("vga"), clock_domain="vga")
            if with_video_terminal:
                self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga")
            if with_video_framebuffer:
                self.add_video_framebuffer(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga", format="rgb565")
            if with_video_colorbars:
                self.add_video_colorbars(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.submodules.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

        if (with_daughterboard):
            from litex_boards.platforms.qmtech_daughterboard import SevenSeg
            self.submodules.sevenseg = SevenSeg(
                    segs         = platform.request_all("seven_seg"),
                    sels         = platform.request_all("seven_seg_ctl"),
                    sys_clk_freq = sys_clk_freq)

        if not with_daughterboard and kwargs["uart_name"] == "serial":
            kwargs["uart_name"] = "jtag_serial"

# Build --------------------------------------------------------------------------------------------

def main():
    parser = argparse.ArgumentParser(description="LiteX SoC on QMTech XC7K325T")
    parser.add_argument("--toolchain",           default="vivado",                 help="FPGA toolchain (vivado, symbiflow or yosys+nextpnr).")
    parser.add_argument("--build",               action="store_true",              help="Build bitstream.")
    parser.add_argument("--load",                action="store_true",              help="Load bitstream.")
    parser.add_argument("--sys-clk-freq",        default=100e6,                    help="System clock frequency.")
    parser.add_argument("--with-daughterboard",  action="store_true",              help="Board plugged into the QMTech daughterboard.")
    ethopts = parser.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",      action="store_true",              help="Enable Ethernet support.")
    ethopts.add_argument("--with-etherbone",     action="store_true",              help="Enable Etherbone support.")
    parser.add_argument("--eth-ip",              default="192.168.1.50", type=str, help="Ethernet/Etherbone IP address.")
    parser.add_argument("--eth-dynamic-ip",      action="store_true",              help="Enable dynamic Ethernet IP addresses setting.")
    parser.add_argument("--remote-ip",           default="192.168.1.100",
   help="Remote IP address of TFTP server.")
    parser.add_argument("--local-ip",            default="192.168.1.50",
   help="Local IP address.")
    sdopts = parser.add_mutually_exclusive_group()
    sdopts.add_argument("--with-spi-sdcard",     action="store_true",              help="Enable SPI-mode SDCard support.")
    sdopts.add_argument("--with-sdcard",         action="store_true",              help="Enable SDCard support.")
    parser.add_argument("--with-jtagbone",       action="store_true",              help="Enable Jtagbone support.")
    parser.add_argument("--with-spi-flash",      action="store_true",              help="Enable SPI Flash (MMAPed).")
    viopts = parser.add_mutually_exclusive_group()
    viopts.add_argument("--with-video-terminal",    action="store_true", help="Enable Video Terminal (VGA).")
    viopts.add_argument("--with-video-framebuffer", action="store_true", help="Enable Video Framebuffer (VGA).")
    viopts.add_argument("--with-video-colorbars", action="store_true", help="Enable Video Colorbars (VGA).")
    builder_args(parser)
    soc_core_args(parser)
    vivado_build_args(parser)
    args = parser.parse_args()

    soc = BaseSoC(
        toolchain              = args.toolchain,
        sys_clk_freq           = int(float(args.sys_clk_freq)),
        with_daughterboard     = args.with_daughterboard,
        with_ethernet          = args.with_ethernet,
        with_etherbone         = args.with_etherbone,
        eth_ip                 = args.eth_ip,
        eth_dynamic_ip         = args.eth_dynamic_ip,
        local_ip               = args.local_ip,
        remote_ip              = args.remote_ip,
        with_jtagbone          = args.with_jtagbone,
        with_spi_flash         = args.with_spi_flash,
        with_video_terminal    = args.with_video_terminal,
        with_video_framebuffer = args.with_video_framebuffer,
        with_video_colorbars = args.with_video_colorbars,
        **soc_core_argdict(args)
    )

    if args.with_spi_sdcard:
        soc.add_spi_sdcard()
    if args.with_sdcard:
        soc.add_sdcard()

    builder = Builder(soc, **builder_argdict(args))
    if args.with_ethernet or args.with_etherbone:
        os.makedirs(os.path.join(builder.software_dir, "include/generated"),
                    exist_ok=True)
        write_to_file(
            os.path.join(builder.software_dir, "include/generated", "target.h"),
            "// Force 100Base-T speed\n"
            "#define TARGET_ETHPHY_INIT_FUNC() mdio_write(0, 0, 0x2100)")

    builder_kwargs = vivado_build_argdict(args) if args.toolchain == "vivado" else {}
    if args.build:
	    builder.build(**builder_kwargs)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(os.path.join(builder.gateware_dir, soc.build_name + ".bit"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Andrew Dennison <andrew@motec.com.au>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.openfpgaloader import OpenFPGALoader

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk33", 0, Pins("C3"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),  # net PLL_IN

    # Buttons
    ("user_btn", 0, Pins("C5"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 1, Pins("C9"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),

    # Leds
    ("user_led", 0, Pins("B3"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 1, Pins("J6"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 2, Pins("D7"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 3, Pins("D8"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("F3")),  # net TXD
        Subsignal("rx", Pins("H2")),  # net RXD
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
     ),

    # SPIFlash (W25Q128JVSIM)
    ("spiflash", 0,
        Subsignal("cs_n", Pins("J4")),  # net SPI_SS
        Subsignal("clk",  Pins("H4")),  # net SPI_SCLK
        Subsignal("mosi", Pins("F4")),  # net SPI_MOSI
        Subsignal("miso", Pins("H3")),  # net SPI_MISO
        #Subsignal("wp",   Pins("")),
        #Subsignal("hold", Pins("")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
     ),

    # SDCard
    ("spisdcard", 0,
        # All 4 SPI signals below have 10k pullup on dev board
        Subsignal("clk",  Pins("J2")),  # net SD_SCLK
        Subsignal("cs_n", Pins("G5")),  # net SD_CS
        Subsignal("mosi", Pins("G4")),  # net SD_DI
        Subsignal("miso", Pins("J3")),  # net SD_DO
        Subsignal("det",  Pins("G3")),  # net CD2
        Misc("SLEW=FAST"),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
     ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    # Wacky PMOD pinout misunderstood by others too
    # resulting in this order: GPIOL_12,14,16,19,13,15,17,18
    ("pmod", "G1 E2 C2 D3 F1 E1 D2 E3"),
    # GPIOR_020..28, 30..32, 34..36
    ("j1", "F8 E7 F7 E6 F6 F5 G9 H9 J9 J8 G8 H8 J7 G6 H6"),
    # GPIOR_19..10, 08..05, 03, 01, 00
    ("j2", "E8 D9 D8 D7 C9 B9 D6 C8 B8 A9 A8 C7 C6 B6 A6 B5 A5"),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name = "clk33"
    default_clk_period = 1e9/33.333e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "T8F81C2", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return OpenFPGALoader("xyloni_spi")

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk33", loose=True), 1e9/33.333e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Antmicro <www.antmicro.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    ("clk100", 0, Pins("C12"), IOStandard("LVCMOS33")),

    ("user_led", 0, Pins("D21"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("B20"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("B21"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("C22"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("E22"), IOStandard("LVCMOS33")),

    ("user_btn", 0, Pins("C21"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("A20"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("E21"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("D23"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("E26")),
        Subsignal("rx", Pins("F25")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("C23")),
        Subsignal("dq",   Pins("B24", "A25", "B22", "A22")),
        IOStandard("LVCMOS33")
    ),

    # DDR4
    ("ddr4", 0,
        Subsignal("a",       Pins(
            "AF10 AC11 AD11 AD10 AC9  AD9  AB9  AF7  AE8",
            "AE7  Y12  AC7  AB7  AD13"),
            IOStandard("SSTL12_DCI")),
        Subsignal("ba",      Pins("AB11 AB10"), IOStandard("SSTL12_DCI")),
        Subsignal("bg",      Pins("AA9 AF9"), IOStandard("SSTL12_DCI")),
        Subsignal("ras_n",   Pins("AA12"), IOStandard("SSTL12_DCI")), # A16
        Subsignal("cas_n",   Pins("AF13"), IOStandard("SSTL12_DCI")), # A15
        Subsignal("we_n",    Pins("AA13"), IOStandard("SSTL12_DCI")), # A14
        Subsignal("cs_n",    Pins("W13"), IOStandard("SSTL12_DCI")),
        Subsignal("act_n",   Pins("Y8"), IOStandard("SSTL12_DCI")),
        Subsignal("alert_n", Pins("AE10"), IOStandard("SSTL12_DCI")),
        Subsignal("par",     Pins("AE13"), IOStandard("SSTL12_DCI")),
        Subsignal("dq",      Pins(
                "W11  Y11  V7   Y7   V11  V9   V8   W8",
                "U2   V6   Y2   Y3   U5   U4   W3   Y1",
                "AA2  AB2  AE1  AE2  V2   W1   AD1  AC2",
                "W4   AA3  AD3  AC4  V3   V4   AB4  AC3",
                "AC16 AC17 AB16 AA19 AB15 AD16 AC18 AC19",
                "AF17 AE17 AF20 AD19 AE15 AE16 AF19 AD18",
                "Y18  Y17  W14  V14  AA20 AA15 V18  W16",
                "AA18 AB19 V16  W15  AB17 AA17 V19  V17"),
            IOStandard("SSTL12_T_DCI")),
        Subsignal("dqs_p",   Pins(
                "W10  B17 W6   D19 AB1 L19 AA5 J15",
                "AD20 T24 AE18 P19 W18 R16 Y15 M25"),
            IOStandard("DIFF_HSUL_12")),
        Subsignal("dqs_n",   Pins(
                "W9   A17 W5   D20 AC1 L20 AB5 J16",
                "AE20 T25 AF18 P20 W19 R17 Y16 L25"),
            IOStandard("DIFF_HSUL_12")),
        Subsignal("clk_p",   Pins("AE12"), IOStandard("DIFF_SSTL12_DCI")),
        Subsignal("clk_n",   Pins("AF12"), IOStandard("DIFF_SSTL12_DCI")),
        Subsignal("cke",     Pins("AA8"), IOStandard("SSTL12_DCI")), # also AM15 for larger SODIMMs
        Subsignal("odt",     Pins("Y13"), IOStandard("SSTL12_DCI")), # also AM16 for larger SODIMMs
        Subsignal("reset_n", Pins("AB6"), IOStandard("SSTL12")),
        Misc("SLEW=FAST"),
    ),

    # RGMII Ethernet
    ("eth_ref_clk", 0, Pins("AA23"), IOStandard("LVCMOS33")),
    ("eth_clocks", 0,
        Subsignal("rx", Pins("Y23")),
        Subsignal("tx", Pins("AA24")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("AA22")),
        Subsignal("mdio",    Pins("AB26")),
        Subsignal("mdc",     Pins("AA25")),
        Subsignal("rx_ctl",  Pins("Y25")),
        Subsignal("rx_data", Pins("W26 W25 V26 U25")),
        Subsignal("tx_ctl",  Pins("U26")),
        Subsignal("tx_data", Pins("W24 Y26 Y22 Y21")),
        IOStandard("LVCMOS33")
    ),

    # HyperRAM
    ("hyperram", 0,
        Subsignal("clk", Pins("AD26")), # clk_n AE26
        Subsignal("rst_n", Pins("AC24")),
        Subsignal("cs_n",  Pins("AC26")),
        Subsignal("dq",    Pins("AE23 AD25 AF24 AE22 AF23 AF25 AE25 AD24")),
        Subsignal("rwds",  Pins("AD23")),
        IOStandard("LVCMOS33")
    ),

    # SD Card
    ("sdcard", 0,
        Subsignal("data", Pins("E10 F8 C9 D9"), Misc("PULLUP True")),
        Subsignal("cmd",  Pins("D8"), Misc("PULLUP True")),
        Subsignal("clk",  Pins("D10")),
        Subsignal("cd",   Pins("F9")),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS33"),
    ),

    # I2C
    ("i2c", 0,
        Subsignal("scl", Pins("E25")),
        Subsignal("sda", Pins("D26")),
        IOStandard("LVCMOS33"),
    ),

    # HDMI Out
    ("hdmi_out", 0,
        Subsignal("clk_p",   Pins("B15"),   IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("A15"),   IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("B14"),   IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("A14"),   IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("A13"),  IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("A12"),  IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("B10"),  IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("A10"),  IOStandard("TMDS_33")),
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, device="xc7k160tffg676-1", toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, device, _io, toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.6 [get_iobanks 32]")
        self.add_platform_command("set_property INTERNAL_VREF 0.6 [get_iobanks 33]")
        self.add_platform_command("set_property INTERNAL_VREF 0.6 [get_iobanks 34]")
        self.add_platform_command("set_property DCI_CASCADE {{32 34}} [get_iobanks 33]")

    def create_programmer(self):
        bscan_spi = "bscan_spi_xc7k160t.bit" if "xc7k160t" in self.device else "bscan_spi_xc7k160t.bit"
        return OpenOCD("openocd_xc7_ft4232.cfg", bscan_spi)

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Antony Pavlov <antonynpavlov@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk50", 0, Pins("AF14"), IOStandard("3.3-V LVTTL")),
    ("clk50", 1, Pins("AA16"), IOStandard("3.3-V LVTTL")),
    ("clk50", 2, Pins("Y26"),  IOStandard("3.3-V LVTTL")),
    ("clk50", 3, Pins("K14"),  IOStandard("3.3-V LVTTL")),

    # Leds
    ("user_led", 0, Pins("V16"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("W16"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("V17"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("V18"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("W17"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("W19"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("Y19"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("W20"), IOStandard("3.3-V LVTTL")),
    ("user_led", 8, Pins("W21"), IOStandard("3.3-V LVTTL")),
    ("user_led", 9, Pins("Y21"), IOStandard("3.3-V LVTTL")),

    # Seven Segment
    ("seven_seg", 0, Pins("AE26 AE27 AE28 AG27 AF28 AG28 AH28"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 1, Pins("AJ29 AH29 AH30 AG30 AF29 AF30 AD27"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 2, Pins("AB23 AE29 AD29 AC28 AD30 AC29 AC30"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 3, Pins("AD26 AC27 AD25 AC25 AB28 AB25 AB22"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 4, Pins("AA24 Y23  Y24  W22  W24  V23  W25"),  IOStandard("3.3-V LVTTL")),
    ("seven_seg", 5, Pins("V25  AA28 Y27  AB27 AB26 AA26 AA25"), IOStandard("3.3-V LVTTL")),

    # Button
    ("key", 0, Pins("AA14"), IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("AA15"), IOStandard("3.3-V LVTTL")),
    ("key", 2, Pins("W15"),  IOStandard("3.3-V LVTTL")),
    ("key", 3, Pins("Y16"),  IOStandard("3.3-V LVTTL")),

    # Switches
    ("user_sw", 0, Pins("AB12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 1, Pins("AC12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 2, Pins("AF9"),  IOStandard("3.3-V LVTTL")),
    ("user_sw", 3, Pins("AF10"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 4, Pins("AD11"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 5, Pins("AD12"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 6, Pins("AE11"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 7, Pins("AC9"),  IOStandard("3.3-V LVTTL")),
    ("user_sw", 8, Pins("AD10"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 9, Pins("AE12"), IOStandard("3.3-V LVTTL")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("JP1:1"), IOStandard("3.3-V LVTTL")),
        Subsignal("rx", Pins("JP1:2"),  IOStandard("3.3-V LVTTL"))
    ),

    # I2C
    ("i2c", 0,
        Subsignal("sclk", Pins("J12")),
        Subsignal("sdat", Pins("K12")),
        IOStandard("3.3-V LVTTL")
    ),

    # VGA
    ("vga", 0,
        Subsignal("hsync_n", Pins("B11")),
        Subsignal("vsync_n", Pins("D11")),
        Subsignal("r", Pins("A13 C13 E13 B12 C12 D12 E12 F1")),
        Subsignal("g", Pins("J9 J10 H12 G10 G11 G12 F11 E11")),
        Subsignal("b", Pins("B13 G13 H13 F14 H14 F15 G15 J14 F10")),
        IOStandard("3.3-V LVTTL")
    ),

    # GPIOs
    ("gpio_0", 0, Pins(
        "D3 C3  A2  A3  B3  B4  A4  B5",
        "A5 D5  B6  A6  B7  D6  A7  C6",
        "C8 E6  E7  D8  E8  F8  F9  E9",
        "C9 D9 E11 E10 C11 B11 A12 D11",
        "D12 B12"),
        IOStandard("3.3-V LVTTL")
    ),
    ("gpio_1", 0, Pins(
        "F13 T15 T14 T13 R13 T12 R12 T11",
        "T10 R11 P11 R10 N12  P9  N9 N11",
        "L16 K16 R16 L15 P15 P16 R14 N16",
        "N15 P14 L14 N14 M10 L13 J16 K15",
        "J13 J14"),
        IOStandard("3.3-V LVTTL")
    ),

    # SDR SDRAM
    ("sdram_clock", 0, Pins("AH12"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            "AK14 AH14 AG15 AE14 AB15 AC14 AD14 AF15",
            "AH15 AG13 AG12 AH13 AJ14")),
        Subsignal("ba",    Pins("AF13 AJ12")),
        Subsignal("cs_n",  Pins("AG11")),
        Subsignal("cke",   Pins("AK13")),
        Subsignal("ras_n", Pins("AE13")),
        Subsignal("cas_n", Pins("AF11")),
        Subsignal("we_n",  Pins("AA13")),
        Subsignal("dq", Pins(
            "AK6   AJ7 AK7 AK8 AK9 AG10 AK11 AJ11",
            "AH10 AJ10 AJ9 AH9 AH8  AH7  AJ6 AJ5")),
        Subsignal("dm", Pins("AB13 AK12")),
        IOStandard("3.3-V LVTTL")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    # PIN    0 1    2    3    4    5    6    7    8    9    10   11 12 13   14   15   16   17   18   19   20   21   22   23   24   25   26   27   28   29 30 31   32   33   34   35   36   37   38   39   40
    ("JP1", "- AC18 Y17  AD17 Y18  AK16 AK18 AK19 AJ19 AJ17 AJ16 -  -  AH18 AH17 AG16 AE16 AF16 AG17 AA18 AA19 AE17 AC20 AH19 AJ20 AH20 AK21 AD19 AD20 -  -  AE18 AE19 AF20 AF21 AF19 AG21 AF18 AG20 AG18 AJ21"),
    ("JP2", "- AB17 AA21 AB21 AC23 AD24 AE23 AE24 AF25 AF26 AG25 -  -  AG26 AH24 AH27 AJ27 AK29 AK28 AK27 AJ26 AK26 AH25 AJ25 AJ24 AK24 AG23 AK23 AH23 -  -  AK22 AJ22 AH22 AG22 AF24 AF23 AE22 AD21 AA20 AC22")
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, toolchain="quartus"):
        AlteraPlatform.__init__(self, "5CSEMA5F31C6", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return USBBlaster(cable_name="DE-SoC", device_id=2)

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk50", 0, Pins("M9"), IOStandard("3.3-V LVCMOS")),

    # Button
    ("key", 0, Pins("J17"),  IOStandard("3.3-V LVCMOS")),
    ("key", 1, Pins("E16"),  IOStandard("3.3-V LVCMOS")),

    # SPIFlash (MT25QL128ABA)
    ("spiflash", 0,
        # clk
        Subsignal("cs_n", Pins("R4")),
        Subsignal("clk",  Pins("V3")),
        Subsignal("mosi", Pins("AB4")),
        Subsignal("miso", Pins("AB3")),
        IOStandard("3.3-V LVCMOS"),
    ),

    # SDR SDRAM
    ("sdram_clock", 0, Pins("G18"), IOStandard("3.3-V LVCMOS")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            # A0  A1  A2  A3  A4  A5  A6  A7
            "M18 M20 M16 L17 L19 L18 K16 K17",
            # A8  A9 A10 A11 A12
            "J18 J19 N19 H18 H20")),
        Subsignal("ba",    Pins("P19 P18")),
        Subsignal("cs_n",  Pins("P17")),
        Subsignal("cke",   Pins("G17")),
        Subsignal("ras_n", Pins("P16")),
        Subsignal("cas_n", Pins("T19")),
        Subsignal("we_n",  Pins("U20")),
        Subsignal("dq", Pins(
            "AA22 AB22 Y22 Y21 W22 W21 V21 U22 M21 M22 T22 R21 R22 P22 N20 N21 ",
            "K22   K21 J22 J21 H21 G22 G21 F22 E22 E20 D22 D21 C21 B22 A22 B21")),
        Subsignal("dm", Pins("U21 L22 K20 E21")),
        IOStandard("3.3-V LVCMOS")
    ),
]

# The connectors are named after the daughterboard, not the core board
# because on the different core boards the names vary, but on the
# daughterboard they stay the same, which we need to connect the
# daughterboard peripherals to the core board.
# On this board J2 is U6 and J3 is U5
_connectors = [
    ("J2", {
         # odd row   even row
          7: "AB21",   8: "AB20",
          9: "Y19",   10: "Y20",
         11: "AA20",  12: "AA19",
         13: "W19",   14: "V20",
         15: "AB18",  16: "AB17",
         17: "U17",   18: "U16",
         19: "R16",   20: "R17",
         21: "T15",   22: "R15",
         23: "R14",   24: "P14",
         25: "AA15",  26: "AB15",
         27: "T13",   28: "T12",
         29: "R11",   30: "R10",
         31: "AA13",  32: "AA14",
         33: "Y15",   34: "Y14",
         35: "AB12",  36: "AB13",
         37: "AB11",  38: "AB10",
         39: "V10",   40: "V9",
         41: "U12",   42: "U11",
         43: "R9",    44: "T10",
         45: "T8",    46: "T7",
         47: "N8",    48: "P8",
         49: "M7",    50: "M6",
         51: "N6",    52: "P6",
         53: "R5",    54: "R6",
         55: "AB8",   56: "AA8 ",
         57: "AB7",   58: "AA7",
         59: "AB5",   60: "AB6",
    }),
    ("J3", {
        # odd row     even row
         7: "F19",    8: "F18",
         9: "E19",   10: "D19",
        11: "C20",   12: "B20",
        13: "A20",   14: "A19",
        15: "C19",   16: "C18",
        17: "A18",   18: "A17",
        19: "B18",   20: "B17",
        21: "B16",   22: "C16",
        23: "C15",   24: "B15",
        25: "E15",   26: "F15",
        27: "A15",   28: "A14",
        29: "B13",   30: "A13",
        31: "B12",   32: "A12",
        33: "G15",   34: "F14",
        35: "H13",   36: "G13",
        37: "D12",   38: "E12",
        39: "H11",   40: "G12",
        41: "A10",   42: "A9",
        43: "J9",    44: "H9",
        45: "E9",    46: "D9",
        47: "H8",    48: "G8",
        49: "L7",    50: "K7",
        51: "J7",    52: "J8",
        53: "A8",    54: "A7",
        55: "B6",    56: "B7",
        57: "C6",    58: "D6",
        59: "A5",    60: "B5",
    })
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6
    core_resources = [
        ("user_led", 0, Pins("V19"), IOStandard("3.3-V LVCMOS")),
        ("user_led", 1, Pins("T20"), IOStandard("3.3-V LVCMOS")),
        ("serial", 0,
            Subsignal("tx", Pins("J3:7"), IOStandard("3.3-V LVCMOS")),
            Subsignal("rx", Pins("J3:8"), IOStandard("3.3-V LVCMOS"))
        ),
 ]

    def __init__(self, toolchain="quartus", with_daughterboard=False):
        device = "5CEFA5F23I7"
        io = _io
        connectors = _connectors

        if with_daughterboard:
            from litex_boards.platforms.qmtech_daughterboard import QMTechDaughterboard
            daughterboard = QMTechDaughterboard(IOStandard("3.3-V LVCMOS"))
            io += daughterboard.io
            connectors += daughterboard.connectors
        else:
            io += self.core_resources

        AlteraPlatform.__init__(self, device, io, connectors, toolchain=toolchain)

        if with_daughterboard:
            # ethernet takes the config pin, so make it available
            self.add_platform_command("set_global_assignment -name CYCLONEII_RESERVE_NCEO_AFTER_CONFIGURATION \"USE AS REGULAR IO\"")

        # Generate PLL clock in STA
        self.toolchain.additional_sdc_commands.append("derive_pll_clocks")
        # Calculates clock uncertainties
        self.toolchain.additional_sdc_commands.append("derive_clock_uncertainty")

    def create_programmer(self):
        return USBBlaster()

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)
